// Float64Message.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "Message.h"
#include "StandardFieldsMessage.h"
#include "Float64Message.h"

DLList* Float64Message::pFreeList = 0;

int32 Float64Message:: toBuffer(char **buffer) const
{
    int32 nbytes;

    nbytes = StandardFieldsMessage::toBuffer(buffer);
    nbytes += MPF::toBuffer(buffer,value);
    return(nbytes);
}

int32 Float64Message:: fromBuffer(const char **buffer)
{
    int32 nbytes=0;

    nbytes = StandardFieldsMessage::fromBuffer(buffer);
    nbytes += MPF::fromBuffer(buffer,&value);
    return(nbytes);
}

int32 Float64Message:: fromBufferSwitch(const char **buffer)
{
    int32 nbytes=0;

    nbytes = StandardFieldsMessage::fromBufferSwitch(buffer);
    nbytes += MPF::fromBufferSwitch(buffer,&value);
    return(nbytes);
}

void Float64Message:: print() const
{
    StandardFieldsMessage::print();
    printf("value %f\n",value);
}

void *Float64Message:: operator new(size_t size)
{
    if(size==sizeof(Float64Message)) return MPF:: allocate(&pFreeList,size);
    printf("Float64Message new size !=sizeof(Float64Message)\n");
    return(0);
}
 
void Float64Message:: operator delete(void *p, size_t size)
{
    if(size==sizeof(Float64Message)) {
	MPF::free(pFreeList,p,size);
	return;
    }
    printf("Float64Message delete size !=sizeof(Float64Message)\n");
}
