//gpibSniff.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
	Author: Marty Kraimer
	Date: 21OCT98
*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
 
#include <tickLib.h>
#include <taskLib.h>
#include <rngLib.h>
#include <sysLib.h>
 
int gpibRingBufSize = 1000;

#include "Gpib.h"
 
class SniffData {
public:
    bool isReceive;
    unsigned char byte;
};

class SniffPrivate {
public:
    RING_ID ringbuf;
};

static void peakTask(const char *linkName,int nseconds);
static char taskname[] = "gpibSniff";

int gpibSniff(const char *linkName,int nseconds)
{
    taskSpawn(taskname,130,VX_FP_TASK,2000,
	(FUNCPTR)peakTask,
	(int)linkName,nseconds,0,0,0,0,0,0,0,0);
    return(0);
}

static void peakHandler(void* parm,bool isReceive, unsigned char byte)
{
    SniffPrivate *pvt = (SniffPrivate *)parm;
    SniffData data;
    data.isReceive = isReceive;
    data.byte = byte;
    rngBufPut(pvt->ringbuf,(char *)&data,sizeof(SniffData));
}

static void printQueue(bool isReceive, unsigned char *byteQueue,int len)
{
    printf("%c",(isReceive ? 'R' : 'T'));
    for(int i=0;i<len; i++) {
        if(isprint(byteQueue[i])) printf("%c",byteQueue[i]);
        else printf(" 0x%2.2x",byteQueue[i]);
    }
    printf("\n");
}
    
static void peakTask(const char *linkName,int nseconds)
{
    SniffPrivate pvt;
    unsigned char byteQueue[80];
    int nextQueue = 0;

    if((pvt.ringbuf = rngCreate(sizeof(void *) * gpibRingBufSize))==NULL){
	printf("peakTask: initWatchDog failed\n");
	return;
    }
    if(!Gpib::setSniffHandler(linkName,&pvt,peakHandler)) return;
    unsigned int start = (unsigned int )::tickGet();
    bool isReceive = true;
    printf("\n");
    while(((unsigned int)tickGet() - start)
    < (unsigned int)(nseconds*sysClkRateGet())) {
	taskDelay(sysClkRateGet());
	while (rngNBytes(pvt.ringbuf)>=(int)sizeof(SniffData)){
	    SniffData data;
	    if(rngBufGet(pvt.ringbuf,(char *)&data,
	    sizeof(SniffData))!=(int)sizeof(SniffData)) {
		printf("peakTask rngBufGet returned error\n");
	    } else {
                if(data.isReceive!=isReceive || nextQueue==80) {
		    printQueue(isReceive,byteQueue,nextQueue);
		    nextQueue = 0;
		}
                isReceive = data.isReceive;
		byteQueue[nextQueue++] = data.byte;
	    }
	}
    }
    if(nextQueue>0) printQueue(isReceive,byteQueue,nextQueue);
    printf("gpibSniff done\n");
    Gpib::freeSniffHandler(linkName);
    rngDelete(pvt.ringbuf);
}
